/*
 * PS2_Controller.c
 *
 *  Created on: 2015-02-08
 *      Author: Nelson Stoik
 *      Last Modififed: 2015-02-28
 */

#include "PS2_Controller.h"


/* Define test conditions for flow control */
#define READ_AND_DECODE_GOOD		1
#define DECODE_BAD					2
#define READ_BAD					3

#define PS2_INPUT_ENABLED			1
#define	PS2_INPUT_DISABLED			2


#define DECODE_STRING_LENGTH	10						/*max length of decode string for bytes and ASCII letters */

alt_up_ps2_dev	*ps2_device;							/*ps2_device structure used by the ps2 controller */
OS_EVENT		*input_mutex;							/*mutex to protect the input_enabled variable */
OS_EVENT		*wait_for_input_enabled;				/*semaphore to pause scanning until input has been enabled */

static int 	input_enabled;								/*variable to start and stop input */


/*
 * sets all variables to their starting values and starts the ps2 monitoring
 * task. call once before starting the main program
 */
void ps2_initialisation()
{
	INT8U	err;

	printf("Initialising the ps2 device...\n");

	//set up the ps2_device
	ps2_device->base = PS2_0_BASE;
	ps2_device->irq_id = PS2_0_IRQ;
	ps2_device->timeout = 0;
	ps2_device->device_type = PS2_KEYBOARD;

	wait_for_input_enabled = OSSemCreate(0);

	input_mutex = OSMutexCreate(PS2_INPUT_MUTEX_PRIORITY, &err);
	if (err != OS_NO_ERR)
	{
		printf("error creating input_mutex\n");
	}

	input_enabled = PS2_INPUT_ENABLED;

	printf("Initialised\n");
	return;
}

/*
 * clears all stored input from the ps2 controller
 */
void clear_input()
{
	alt_up_ps2_clear_fifo(ps2_device);
}

/*
 * enables the task to scan for data from ps2 device.
 * NOTE: scanning is enabled by default after ps2_initialisation is run
 */
void enable_scanning()
{
	INT8U err;

	OSMutexPend(input_mutex, 0, &err);
	input_enabled = PS2_INPUT_ENABLED;
	OSMutexPost(input_mutex);
	OSSemPost(wait_for_input_enabled);
}
/*
 * disables the task from scanning for data from the ps2 device
 */
void disable_scanning()
{
	INT8U	err;

	OSMutexPend(input_mutex, 0, &err);
	input_enabled = PS2_INPUT_DISABLED;
	OSMutexPost(input_mutex);
}


/*
 * a task that polls the ps2 controller for available input. Is paused by
 * calling disable_scanning() and resumed by calling enable_scanning()
 */
void input_monitoring_task(void* pdata)
{
	//message read from the ps2 controller
	char	message_received[DECODE_STRING_LENGTH];

	int		read_return = READ_BAD;
	INT8U	err;

	ps2_initialisation();

	memset(message_received, 0, DECODE_STRING_LENGTH);


	while (1)
	{

		//reset the variable and attempt to read from PS2
		memset(message_received, 0, DECODE_STRING_LENGTH);
		read_return = read_and_decode(message_received);

		//good read from PS2, parse it
		if (read_return == READ_AND_DECODE_GOOD)
		{
			//ignore blanks; these are the break codes from the keyboard
			if(strncmp(message_received, "",sizeof("")) != 0)
			{
				printf("%s\n", message_received);
			}

		}
		//bad read, wait and try again
		else {
			//printf("bad read\n");
			OSTimeDlyHMSM(0, 0, 0, 500);
		}

		//check if the input has been disabled, wait until it has been enabled again
		OSMutexPend(input_mutex, 0, &err);
		if(input_enabled == PS2_INPUT_DISABLED)
		{
			OSMutexPost(input_mutex);
			OSSemPend(wait_for_input_enabled, 0, &err);
		}
		else
		{
			OSMutexPost(input_mutex);
		}

	}
}
/*
 * read the codes from the ps2 controller and convert them to ascii
 * RETURNS:
 * 			DECODE_BAD: 			bad break code
 * 			READ_BAD:				bad read from ps2
 * 			READ_AND_DECODE_GOOD:	good read and decode
 */
int read_and_decode(char *decoded_string)
{
	alt_u8			byte;
	char			ascii[DECODE_STRING_LENGTH];
	char			translated_string[DECODE_STRING_LENGTH];
	KB_CODE_TYPE 	*decode_mode;
	int 			check_status;

	//clear all variables
	memset(ascii, 0, DECODE_STRING_LENGTH);
	memset(translated_string, 0, DECODE_STRING_LENGTH);
	byte = 0;

	//read the data from the ps2 controller
	check_status = decode_scancode(ps2_device, &decode_mode, &byte, &ascii);

	if (check_status == 0)
	{
		//read from ps2 controller was good, now decode the data depending on decode_mode
		switch ((int) decode_mode)
		{
		case KB_ASCII_MAKE_CODE:
		{
			translate_make_code(KB_ASCII_MAKE_CODE, byte, &translated_string);
			break;
		}
		case KB_BINARY_MAKE_CODE:
		{
			translate_make_code(KB_BINARY_MAKE_CODE, byte, &translated_string);
			break;
		}
		case KB_LONG_BINARY_MAKE_CODE:
		{
			translate_make_code(KB_LONG_BINARY_MAKE_CODE, byte, &translated_string);
			break;
		}
		case KB_BREAK_CODE:
		{
			translate_make_code(KB_BREAK_CODE, byte, &translated_string);
			break;
		}
		case KB_LONG_BREAK_CODE:
		{
			translate_make_code(KB_LONG_BREAK_CODE, byte, &translated_string);
			break;
		}
		default:
		{
			return DECODE_BAD;
		}
		}

		//string has been read and decoded successfully, return good end condition
		strncpy(decoded_string, translated_string, strlen(translated_string));
		return READ_AND_DECODE_GOOD;
	}

	//no data from ps2 controller, return READ_BAD
	else
	{
		return READ_BAD;
	}
	//will never reach here
	return READ_BAD;
}

